% This module does the analaysis for the gasoline dataset - load set params
load GasData;

seed = 8; rand('state',8); randn('state',8);

% Set the parameters we will run this under
N = length(Y);         % Remember the Sample Size
B = [10^4; 10^8];      % The smoothness constraint, first on beta, then on bootstrap
VIT = 5:0.5:10;        % Create a grid for the Ts
k_n = 0;               % The number of knots for Bsplines
M4N = 2;               % This is m + m0, used in constructing norm constraint
Ord_X = 2;             % The order of the Bsplines
BCon = N^(2/3);        % The weight for the Bootstrap penalty
R = 2000;              % Bootstrap Iterations     

bootflag = 1;          % Set bootflag to one if you want bootstrap critical values

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Process Data Ready for Optimizer %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Evaluate the X at the different splines. Knots put at the percentiles
knots = (0:1/(k_n + 1):1)*100; knots = prctile(X,knots); 
knots = augknt(knots, Ord_X); [T_X I] = sort(X,1);
XSplines = spcol(knots, Ord_X, T_X);
T = [I XSplines]; T = sortrows(T,1);
XSplines = T(:,2:end);

clear T_X I T;

% And put the data together
Data = [Y Z XSplines];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Set up some of the constraints %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Moving on to the smoothness constraint, we need to set up a grid of points 
% where to evaluate the derivative of the splines
div4g = 0.01;
T1 = (7:div4g:13)'; T1 = repmat(T1,M4N,1); T1 = sort(T1);
T2 = spcol(knots, Ord_X, T1);
XGrid = ones(length(7:div4g:13), size(XSplines,2), M4N);
for i = 1:M4N
    for j = 1:length(7:div4g:13)
        XGrid(j,:,i) = T2(M4N*(j-1) + i,:);
    end
end

% Now with this grid we'll do numerical integration
LM = zeros(size(XGrid,2));
for d = 1:size(XGrid,3)
    for j = 1:size(XGrid,2)
        T = (repmat(XGrid(:,j,d)', size(XGrid,2), 1))*XGrid(:,:,d);
        LM(j,:) = LM(j,:) + trapz(T)*div4g;
    end
end

clear T T1 T2 i j ZGrid d div4g;

% Finish setting up the constraints
Am = []; Bm = []; LB = []; UB = []; 
Aeq = []; BootAeq = []; Beq = []; BootBeq = [];

% And optimization options - for full sample problem
options = optimset('fmincon'); options.GradObj = 'on'; 
options.Display = 'off'; options.GradConstr = 'on'; 
options.DerivativeCheck = 'off'; options.MaxFunEvals = 10000; 
options.MaxIter = 10000; options.LargeScale = 'off'; 
options.Algorithm = 'active-set'; options.TolFun = 1e-8;

% Set the first starting values
x0 = zeros(2,1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Run linear and bootstrap %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
% Set up the functions to be called
fobj = @(x)f_bier_minmax(x,Data,VIT);
fcon = @(x)f_bier_nonlcon(x,LM,B(1));
    
% And finally call the optimizer
[argopt Temp2 xflag] = fminimax(fobj, x0, Am, Bm, Aeq, Beq, LB, UB, fcon, options);
opt = max(Temp2);        

if bootflag == 1
    % Calculate the bootstrap statistic. Get sample, etc.
    BI = int64(1 + rand(N,R)*(N-1));
    BVal = zeros(R,1); BXflag = zeros(R,1);
    
    % Now get the bootstrap critical value - but only if you converged
    % Set the starting value for the bootstrap
    x0 = zeros(size(XSplines,2)*2,1);

    parfor b = 1:R
        %Put together the bootstrap sampler
        BY = Y(BI(:,b)); BZ = Z(BI(:,b));
            
        % And the Xsplines
        BXS = XSplines*0;
        for c = 1:size(XSplines,2)
            T = XSplines(:,c);
            BXS(:,c) = T(BI(:,b));
        end            
    
        BData = [BY BZ BXS; Data];
            
        % Set up the functions we will call
        fbootobj = @(x)f_boot_minmax(x,BData,VIT,BCon);
        fbootcon = @(x)f_boot_nonlcon(x,LM,B);
            
        % Call the optimizer
        [T Temp BXflag(b)] = fminimax(fbootobj, x0, Am, Bm, BootAeq, BootBeq, LB, UB, fbootcon, options);           
        BVal(b) = max(Temp);                              
    end        
        
    % Get the quantile we are looking for
    BVal = BVal(BXflag > 0);
    quant = sum(BVal <= opt)/length(BVal);                           
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Do also a standard J-Test %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Set up the data
ZGMM = normpdf((repmat(Z, 1, length(VIT)) - repmat(VIT,length(Z),1))/0.05);
ZGMM = [ZGMM normpdf((repmat(Z, 1, length(VIT)) - repmat(VIT,length(Z),1))/0.1)];
ZGMM = [ZGMM normpdf((repmat(Z, 1, length(VIT)) - repmat(VIT,length(Z),1))/0.2)];
XGMM = [ones(length(X),1) X];

% Compute first stage estimator
Sxz = XGMM'*ZGMM/length(X);
Sxy = ZGMM'*Y/length(X);
g1GMM = inv(Sxz*Sxz')*Sxz*Sxy;

% Find the optimal weighting matrix and estimator
W = inv((repmat((Y - XGMM*g1GMM),1,size(ZGMM,2)).*ZGMM)'*(repmat((Y - XGMM*g1GMM),1,size(ZGMM,2)).*ZGMM)/length(Y));
g2GMM = inv(Sxz*W*Sxz')*Sxz*W*Sxy;

% And compute the Jtest
J = length(Y)*mean(repmat((Y - XGMM*g2GMM),1,size(ZGMM,2)).*ZGMM)*W*mean(repmat((Y - XGMM*g2GMM),1,size(ZGMM,2)).*ZGMM)'
pval = 1 - chi2cdf(J,size(ZGMM,2)-2)